@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,6 +23,14 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include <osmium/io/file.hpp>
+#include <osmium/io/writer_options.hpp>
+#include <osmium/osm/entity_bits.hpp>
+#include <osmium/util/file.hpp>
+#include <osmium/util/verbose_output.hpp>
+
+#include <boost/program_options.hpp>
+
 #include <functional>
 #include <map>
 #include <memory>
@@ -30,29 +38,28 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <utility>
 #include <vector>
 
-#include <boost/program_options.hpp>
-namespace po = boost::program_options;
-
-#include <osmium/io/file.hpp>
-#include <osmium/io/writer_options.hpp>
-#include <osmium/osm/entity_bits.hpp>
-#include <osmium/util/file.hpp>
-#include <osmium/util/verbose_output.hpp>
-
 const char* get_osmium_version() noexcept;
 const char* get_osmium_long_version() noexcept;
 const char* get_libosmium_version() noexcept;
 
-namespace osmium { namespace io {
-    class Header;
-}}
+namespace osmium {
+
+    namespace io {
+        class Header;
+    } // namespace io
+
+} // namespace osmium
+
+class CommandFactory;
+
+namespace po = boost::program_options;
 
 /**
  * Virtual base class for commands that can be called from the command line.
  */
 class Command {
 
-    osmium::osm_entity_bits::type m_osm_entity_bits {osmium::osm_entity_bits::all};
+    osmium::osm_entity_bits::type m_osm_entity_bits = osmium::osm_entity_bits::all;
 
     enum class display_progress_type {
         never  = 0,
@@ -62,11 +69,14 @@ class Command {
 
 protected:
 
-    osmium::util::VerboseOutput m_vout {false};
+    const CommandFactory& m_command_factory;
+    osmium::VerboseOutput m_vout{false};
 
 public:
 
-    Command() = default;
+    explicit Command(const CommandFactory& command_factory) :
+        m_command_factory(command_factory) {
+    }
 
     virtual ~Command() {
     }
@@ -106,7 +116,7 @@ public:
     void setup_progress(const boost::program_options::variables_map& vm);
     void setup_object_type_nwrc(const boost::program_options::variables_map& vm);
     void setup_object_type_nwr(const boost::program_options::variables_map& vm);
-    void show_object_types(osmium::util::VerboseOutput& vout);
+    void show_object_types(osmium::VerboseOutput& vout);
     void print_arguments(const std::string& command);
     void show_memory_used();
 
@@ -117,7 +127,7 @@ public:
     bool display_progress() const {
         switch (m_display_progress) {
             case display_progress_type::on_tty:
-                return osmium::util::isatty(2); // if STDERR is a TTY
+                return osmium::isatty(1) && osmium::isatty(2); // if STDOUT and STDERR are a TTY
             case display_progress_type::always:
                 return true;
             default:
@@ -142,7 +152,7 @@ public:
 
     po::options_description add_single_input_options();
 
-    void show_single_input_arguments(osmium::util::VerboseOutput& vout);
+    void show_single_input_arguments(osmium::VerboseOutput& vout);
 
     const osmium::io::File& input_file() const {
         return m_input_file;
@@ -164,7 +174,7 @@ public:
 
     po::options_description add_multiple_inputs_options();
 
-    void show_multiple_inputs_arguments(osmium::util::VerboseOutput& vout);
+    void show_multiple_inputs_arguments(osmium::VerboseOutput& vout);
 
     const std::vector<osmium::io::File>& input_files() const {
         return m_input_files;
@@ -197,7 +207,7 @@ public:
 
     po::options_description add_output_options();
 
-    void show_output_arguments(osmium::util::VerboseOutput& vout);
+    void show_output_arguments(osmium::VerboseOutput& vout);
 
     const osmium::io::File& output_file() const {
         return m_output_file;
@@ -229,33 +239,23 @@ class CommandFactory {
 
     std::map<const std::string, command_info> m_commands;
 
-    // The constructor is private because CommandFactory is a singleton
-    CommandFactory() :
-        m_commands() {
-    }
-
 public:
 
-    // CommandFactory is a singleton, this returns the only instance
-    static CommandFactory& instance();
+    bool register_command(const std::string& name, const std::string& description, create_command_type&& create_function);
 
     // Return a vector with names and descriptions of all commands
-    static std::vector<std::pair<std::string, std::string>> help();
+    std::vector<std::pair<std::string, std::string>> help() const;
 
-    static int max_command_name_length();
+    int max_command_name_length() const;
 
-    static bool add(const std::string& name, const std::string& description, create_command_type create_function);
-
-    static std::string get_description(const std::string& name);
-
-    bool register_command(const std::string& name, const std::string& description, create_command_type create_function);
+    std::string get_description(const std::string& name) const;
 
     // This will create a C++ command object from the given name and
     // return it wrapped in a unique_ptr.
-    std::unique_ptr<Command> create_command(const std::string& name);
+    std::unique_ptr<Command> create_command(const std::string& name) const;
 
 }; // class CommandFactory
 
-void register_commands();
+void register_commands(CommandFactory& cmd_factory);
 
 #endif // CMD_HPP
