@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,32 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "command_export.hpp"
+#include "exception.hpp"
+#include "util.hpp"
+
+#include "export/export_format_json.hpp"
+#include "export/export_format_text.hpp"
+#include "export/export_handler.hpp"
+
+#include <osmium/area/assembler.hpp>
+#include <osmium/area/multipolygon_manager.hpp>
+#include <osmium/handler/check_order.hpp>
+#include <osmium/io/any_input.hpp>
+#include <osmium/io/reader_with_progress_bar.hpp>
+#include <osmium/memory/buffer.hpp>
+#include <osmium/osm.hpp>
+#include <osmium/relations/manager_util.hpp>
+#include <osmium/util/string.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
+
+#include <rapidjson/document.h>
+#include <rapidjson/error/en.h>
+#include <rapidjson/istreamwrapper.h>
+
+#include <boost/program_options.hpp>
+
 #include <cctype>
 #include <fstream>
 #include <iostream>
@@ -28,30 +54,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <utility>
 #include <vector>
 
-#include <boost/program_options.hpp>
-
-#include <rapidjson/document.h>
-#include <rapidjson/error/en.h>
-#include <rapidjson/istreamwrapper.h>
-
-#include <osmium/area/assembler.hpp>
-#include <osmium/area/multipolygon_manager.hpp>
-#include <osmium/handler/check_order.hpp>
-#include <osmium/io/any_input.hpp>
-#include <osmium/memory/buffer.hpp>
-#include <osmium/osm.hpp>
-#include <osmium/relations/manager_util.hpp>
-#include <osmium/util/verbose_output.hpp>
-#include <osmium/visitor.hpp>
-
-#include "command_export.hpp"
-#include "exception.hpp"
-#include "util.hpp"
-
-#include "export/export_handler.hpp"
-#include "export/export_format_json.hpp"
-#include "export/export_format_text.hpp"
-
 static std::string get_attr_string(const rapidjson::Value& object, const char* key) {
     const auto it = object.FindMember(key);
     if (it == object.MemberEnd()) {
@@ -86,6 +88,53 @@ void CommandExport::parse_options(const rapidjson::Value& attributes) {
     m_options.way_nodes = get_attr_string(attributes, "way_nodes");
 }
 
+static Ruleset parse_tags_ruleset(const rapidjson::Value& object, const char* key) {
+    Ruleset ruleset;
+
+    const auto json = object.FindMember(key);
+    if (json == object.MemberEnd() || json->value.IsNull()) {
+        // When this is not set, the default is "other". This is later
+        // changed to "any" if both linear_tags and area_tags are missing.
+        ruleset.set_rule_type(tags_filter_rule_type::other);
+        return ruleset;
+    }
+
+    if (json->value.IsFalse()) {
+        ruleset.set_rule_type(tags_filter_rule_type::none);
+        return ruleset;
+    }
+
+    if (json->value.IsTrue()) {
+        ruleset.set_rule_type(tags_filter_rule_type::any);
+        return ruleset;
+    }
+
+    if (!json->value.IsArray()) {
+        throw config_error{std::string{"'"} + key + "' member in top-level object must be false, true, null, or an array."};
+    }
+
+    if (json->value.GetArray().Empty()) {
+        std::cerr << "Warning! An empty array for 'linear_tags' or 'area_tags' matches any tags.\n"
+                  << "         Please use 'true' instead of the array.\n";
+        ruleset.set_rule_type(tags_filter_rule_type::any);
+        return ruleset;
+    }
+
+    ruleset.set_rule_type(tags_filter_rule_type::list);
+
+    for (const auto& value : json->value.GetArray()) {
+        if (!value.IsString()) {
+            throw config_error{std::string{"Array elements in '"} + key + "' must be strings."};
+        }
+
+        if (value.GetString()[0] != '\0') {
+            ruleset.add_rule(value.GetString());
+        }
+    }
+
+    return ruleset;
+}
+
 static bool parse_string_array(const rapidjson::Value& object, const char* key, std::vector<std::string>& result) {
     const auto json = object.FindMember(key);
     if (json == object.MemberEnd()) {
@@ -131,8 +180,14 @@ void CommandExport::parse_config_file() {
         parse_options(json_attr->value);
     }
 
-    parse_string_array(doc, "linear_tags", m_linear_tags);
-    parse_string_array(doc, "area_tags", m_area_tags);
+    m_linear_ruleset = parse_tags_ruleset(doc, "linear_tags");
+    m_area_ruleset   = parse_tags_ruleset(doc, "area_tags");
+
+    if (m_linear_ruleset.rule_type() == tags_filter_rule_type::other &&
+        m_area_ruleset.rule_type() == tags_filter_rule_type::other) {
+        m_linear_ruleset.set_rule_type(tags_filter_rule_type::any);
+        m_area_ruleset.set_rule_type(tags_filter_rule_type::any);
+    }
 
     parse_string_array(doc, "include_tags", m_include_tags);
     parse_string_array(doc, "exclude_tags", m_exclude_tags);
@@ -168,11 +223,13 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     ("add-unique-id,u", po::value<std::string>(), "Add unique id to each feature ('counter' or 'type_id')")
     ("config,c", po::value<std::string>(), "Config file")
     ("fsync", "Call fsync after writing file")
+    ("geometry-types", po::value<std::string>(), "Geometry types that should be written (default: 'point,linestring,polygon')")
     ("index-type,i", po::value<std::string>()->default_value(default_index_type), "Index type to use")
     ("keep-untagged,n", "Keep features that don't have any tags")
     ("output,o", po::value<std::string>(), "Output file (default: STDOUT)")
     ("output-format,f", po::value<std::string>(), "Output format (default depends on output file suffix)")
     ("overwrite,O", "Allow existing output file to be overwritten")
+    ("print-default-config,C", "Print default config on STDOUT")
     ("show-errors,e", "Output any geometry errors on STDOUT")
     ("stop-on-error,E", "Stop on the first error encountered")
     ("show-index-types,I", "Show available index types")
@@ -200,6 +257,27 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     po::store(po::command_line_parser(arguments).options(parsed_options).positional(positional).run(), vm);
     po::notify(vm);
 
+    if (vm.count("print-default-config")) {
+        std::cout << R"({
+    "attributes": {
+        "type":      false,
+        "id":        false,
+        "version":   false,
+        "changeset": false,
+        "timestamp": false,
+        "uid":       false,
+        "user":      false,
+        "way_nodes": false
+    },
+    "linear_tags":  true,
+    "area_tags":    true,
+    "exclude_tags": [],
+    "include_tags": []
+}
+)";
+        return false;
+    }
+
     if (vm.count("show-index-types")) {
         for (const auto& map_type : map_factory.map_types()) {
             std::cout << map_type << '\n';
@@ -209,6 +287,7 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     }
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_file(vm);
 
     if (vm.count("config")) {
@@ -237,6 +316,27 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
         m_fsync = osmium::io::fsync::yes;
     }
 
+    if (vm.count("geometry-types")) {
+        m_geometry_types.clear();
+        const auto types = osmium::split_string(vm["geometry-types"].as<std::string>(), ',');
+        for (const auto& type : types) {
+            if (type == "point") {
+                m_geometry_types.point = true;
+            } else if (type == "linestring") {
+                m_geometry_types.linestring = true;
+            } else if (type == "polygon") {
+                m_geometry_types.polygon = true;
+            } else if (type == "multipolygon") {
+                m_geometry_types.polygon = true;
+            } else {
+                throw argument_error{"Unknown geometry type in --geometry-types option: " + type + "."};
+            }
+        }
+        if (m_geometry_types.empty()) {
+            throw argument_error{"No geometry types in --geometry-types option."};
+        }
+    }
+
     if (vm.count("index-type")) {
         m_index_type_name = vm["index-type"].as<std::string>();
         if (m_index_type_name != "none" && !map_factory.has_map_type(m_index_type_name)) {
@@ -302,12 +402,30 @@ bool CommandExport::setup(const std::vector<std::string>& arguments) {
     return true;
 }
 
-static void print_taglist(osmium::util::VerboseOutput& vout, const std::vector<std::string>& strings) {
+static void print_taglist(osmium::VerboseOutput& vout, const std::vector<std::string>& strings) {
     for (const auto& str : strings) {
         vout << "    " << str << '\n';
     }
 }
 
+static void print_ruleset(osmium::VerboseOutput& vout, const Ruleset& ruleset) {
+    switch (ruleset.rule_type()) {
+        case tags_filter_rule_type::none:
+            vout << "none\n";
+            break;
+        case tags_filter_rule_type::any:
+            vout << "any\n";
+            break;
+        case tags_filter_rule_type::list:
+            vout << "one of the following:\n";
+            print_taglist(vout, ruleset.tags());
+            break;
+        case tags_filter_rule_type::other:
+            vout << "if other tag list doesn't match\n";
+            break;
+    }
+}
+
 static const char* print_unique_id_type(unique_id_type unique_id) {
     switch (unique_id) {
         case unique_id_type::counter:
@@ -344,10 +462,10 @@ void CommandExport::show_arguments() {
     m_vout << "    user:      " << (m_options.user.empty()      ? "(omitted)" : m_options.user)      << '\n';
     m_vout << "    way_nodes: " << (m_options.way_nodes.empty() ? "(omitted)" : m_options.way_nodes) << '\n';
 
-    m_vout << "  linear tags:\n";
-    print_taglist(m_vout, m_linear_tags);
-    m_vout << "  area tags:\n";
-    print_taglist(m_vout, m_area_tags);
+    m_vout << "  linear tags: ";
+    print_ruleset(m_vout, m_linear_ruleset);
+    m_vout << "  area tags:   ";
+    print_ruleset(m_vout, m_area_ruleset);
 
     if (!m_include_tags.empty()) {
         m_vout << "  include only these tags:\n";
@@ -383,34 +501,40 @@ bool CommandExport::run() {
     osmium::area::Assembler::config_type assembler_config;
     osmium::area::MultipolygonManager<osmium::area::Assembler> mp_manager{assembler_config};
 
-    m_vout << "First pass through input file (reading relations)...\n";
+    m_vout << "First pass (of two) through input file (reading relations)...\n";
     osmium::relations::read_relations(m_input_file, mp_manager);
     m_vout << "First pass done.\n";
 
-    m_vout << "Second pass through input file...\n";
+    m_vout << "Second pass (of two) through input file...\n";
+
+    m_linear_ruleset.init_filter();
+    m_area_ruleset.init_filter();
 
     auto handler = create_handler(m_output_format, m_output_filename, m_output_overwrite, m_fsync, m_options);
-    ExportHandler export_handler{std::move(handler), m_linear_tags, m_area_tags, m_show_errors, m_stop_on_error};
+    ExportHandler export_handler{std::move(handler), m_linear_ruleset, m_area_ruleset, m_geometry_types, m_show_errors, m_stop_on_error};
     osmium::handler::CheckOrder check_order_handler;
 
     if (m_index_type_name == "none") {
-        osmium::io::Reader reader{m_input_file};
+        osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_file};
         osmium::apply(reader, check_order_handler, export_handler, mp_manager.handler([&export_handler](osmium::memory::Buffer&& buffer) {
             osmium::apply(buffer, export_handler);
         }));
         reader.close();
     } else {
         const auto& map_factory = osmium::index::MapFactory<osmium::unsigned_object_id_type, osmium::Location>::instance();
-        auto location_index = map_factory.create_map(m_index_type_name);
-        location_handler_type location_handler{*location_index};
+        auto location_index_pos = map_factory.create_map(m_index_type_name);
+        auto location_index_neg = map_factory.create_map(m_index_type_name);
+        location_handler_type location_handler{*location_index_pos, *location_index_neg};
         location_handler.ignore_errors();
 
-        osmium::io::Reader reader{m_input_filename};
+        osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_filename};
         osmium::apply(reader, check_order_handler, location_handler, export_handler, mp_manager.handler([&export_handler](osmium::memory::Buffer&& buffer) {
             osmium::apply(buffer, export_handler);
         }));
         reader.close();
-        m_vout << "About " << (location_index->used_memory() / (1024 * 1024)) << " MBytes used for node location index (in main memory or on disk).\n";
+        m_vout << "About "
+               << ((location_index_pos->used_memory() + location_index_neg->used_memory()) / (1024 * 1024))
+               << " MBytes used for node location index (in main memory or on disk).\n";
     }
     m_vout << "Second pass done.\n";
     export_handler.close();
