@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,30 +20,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cstdlib>
-#include <fstream>
-#include <iostream>
-#include <memory>
-#include <string>
-#include <vector>
-
-#include <boost/program_options.hpp>
-
-#include <rapidjson/document.h>
-#include <rapidjson/error/en.h>
-#include <rapidjson/istreamwrapper.h>
-
-#include <osmium/geom/coordinates.hpp>
-#include <osmium/handler/check_order.hpp>
-#include <osmium/io/any_input.hpp>
-#include <osmium/io/header.hpp>
-#include <osmium/io/writer_options.hpp>
-#include <osmium/osm.hpp>
-#include <osmium/osm/box.hpp>
-#include <osmium/util/progress_bar.hpp>
-#include <osmium/util/string.hpp>
-#include <osmium/util/verbose_output.hpp>
-
 #include "command_extract.hpp"
 #include "exception.hpp"
 
@@ -58,25 +34,53 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include "extract/strategy_smart.hpp"
 #include "util.hpp"
 
+#include <osmium/geom/coordinates.hpp>
+#include <osmium/handler/check_order.hpp>
+#include <osmium/io/any_input.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/writer_options.hpp>
+#include <osmium/osm.hpp>
+#include <osmium/osm/box.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/string.hpp>
+#include <osmium/util/verbose_output.hpp>
+
+#include <rapidjson/document.h>
+#include <rapidjson/error/en.h>
+#include <rapidjson/istreamwrapper.h>
+
+#include <boost/program_options.hpp>
+
+#include <cstdlib>
+#include <fstream>
+#include <iostream>
+#include <memory>
+#include <string>
+#include <vector>
+
 namespace {
 
     osmium::Box parse_bbox(const rapidjson::Value& value) {
         if (value.IsArray()) {
-            if (value.Size() == 4) {
-                if (value[0].IsNumber() && value[1].IsNumber() && value[2].IsNumber() && value[3].IsNumber()) {
-                    const osmium::Location bottom_left{value[0].GetDouble(), value[1].GetDouble()};
-                    const osmium::Location top_right{value[2].GetDouble(), value[3].GetDouble()};
-
-                    if (bottom_left < top_right) {
-                        return osmium::Box{bottom_left, top_right};
-                    }
-                    throw config_error{"'bbox' array elements must be in order: left, bottom, right, top."};
-                }
-                throw config_error{"'bbox' array elements must be numbers."};
-            } else {
+            if (value.Size() != 4) {
                 throw config_error{"'bbox' must be an array with exactly four elements."};
             }
-        } else if (value.IsObject()) {
+
+            if (!value[0].IsNumber() || !value[1].IsNumber() || !value[2].IsNumber() || !value[3].IsNumber()) {
+                throw config_error{"'bbox' array elements must be numbers."};
+            }
+
+            const osmium::Location location1{value[0].GetDouble(), value[1].GetDouble()};
+            const osmium::Location location2{value[2].GetDouble(), value[3].GetDouble()};
+
+            osmium::Box box;
+            box.extend(location1);
+            box.extend(location2);
+
+            return box;
+        }
+
+        if (value.IsObject()) {
             const auto left   = value.FindMember("left");
             const auto right  = value.FindMember("right");
             const auto top    = value.FindMember("top");
@@ -97,6 +101,7 @@ namespace {
 
                     throw config_error{"Need 'left' < 'right' and 'bottom' < 'top' in 'bbox' object."};
                 }
+
                 throw config_error{"Members in 'bbox' object must be numbers."};
             }
 
@@ -153,7 +158,7 @@ namespace {
         } else if (file_type == "poly") {
             PolyFileParser parser{buffer, file_name};
             return parser();
-        } else if (file_type == "") {
+        } else if (file_type.empty()) {
             throw config_error{"Could not autodetect file type in '(multi)polygon' object. Add a 'file_type'."};
         }
 
@@ -161,15 +166,17 @@ namespace {
     }
 
     std::size_t parse_multipolygon_object(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
-        std::string file_name{get_value_as_string(value, "file_name")};
-        std::string file_type{get_value_as_string(value, "file_type")};
+        const std::string file_name{get_value_as_string(value, "file_name")};
+        const std::string file_type{get_value_as_string(value, "file_type")};
         return parse_multipolygon_object(directory, file_name, file_type, buffer);
     }
 
     std::size_t parse_polygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
         if (value.IsArray()) {
             return parse_polygon_array(value, buffer);
-        } else if (value.IsObject()) {
+        }
+
+        if (value.IsObject()) {
             return parse_multipolygon_object(directory, value, buffer);
         }
 
@@ -179,7 +186,9 @@ namespace {
     std::size_t parse_multipolygon(const std::string& directory, const rapidjson::Value& value, osmium::memory::Buffer& buffer) {
         if (value.IsArray()) {
             return parse_multipolygon_array(value, buffer);
-        } else if (value.IsObject()) {
+        }
+
+        if (value.IsObject()) {
             return parse_multipolygon_object(directory, value, buffer);
         }
 
@@ -314,21 +323,22 @@ std::unique_ptr<ExtractStrategy> CommandExtract::make_strategy(const std::string
     if (name == "simple") {
         if (m_with_history) {
             throw argument_error{"The 'simple' strategy is not supported for history files."};
-        } else {
-            return std::unique_ptr<ExtractStrategy>(new strategy_simple::Strategy{m_extracts, m_options});
         }
-    } else if (name == "complete_ways") {
+        return std::unique_ptr<ExtractStrategy>(new strategy_simple::Strategy{m_extracts, m_options});
+    }
+
+    if (name == "complete_ways") {
         if (m_with_history) {
             return std::unique_ptr<ExtractStrategy>(new strategy_complete_ways_with_history::Strategy{m_extracts, m_options});
-        } else {
-            return std::unique_ptr<ExtractStrategy>(new strategy_complete_ways::Strategy{m_extracts, m_options});
         }
-    } else if (name == "smart") {
+        return std::unique_ptr<ExtractStrategy>(new strategy_complete_ways::Strategy{m_extracts, m_options});
+    }
+
+    if (name == "smart") {
         if (m_with_history) {
             throw argument_error{"The 'smart' strategy is not supported for history files."};
-        } else {
-            return std::unique_ptr<ExtractStrategy>(new strategy_smart::Strategy{m_extracts, m_options});
         }
+        return std::unique_ptr<ExtractStrategy>(new strategy_smart::Strategy{m_extracts, m_options});
     }
 
     throw argument_error{std::string{"Unknown extract strategy: '"} + name + "'."};
@@ -491,6 +501,10 @@ bool CommandExtract::run() {
         }
     }
 
+    if (m_extracts.empty()) {
+        throw config_error{"No extract specified in config file or on the command line."};
+    }
+
     show_extracts();
 
     m_strategy = make_strategy(m_strategy_name);
