@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,17 +23,17 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cstddef>
-#include <memory>
-#include <string>
-#include <vector>
+#include "cmd.hpp" // IWYU pragma: export
+#include "extract/extract.hpp"
+#include "extract/strategy.hpp"
 
 #include <osmium/memory/buffer.hpp>
 #include <osmium/util/options.hpp>
 
-#include "cmd.hpp" // IWYU pragma: export
-#include "extract/extract.hpp"
-#include "extract/strategy.hpp"
+#include <cstddef>
+#include <memory>
+#include <string>
+#include <vector>
 
 class CommandExtract : public Command, public with_single_osm_input, public with_osm_output {
 
@@ -42,7 +42,7 @@ class CommandExtract : public Command, public with_single_osm_input, public with
     std::string m_config_file_name;
     std::string m_config_directory;
     std::string m_output_directory;
-    osmium::util::Options m_options;
+    osmium::Options m_options;
     std::string m_strategy_name;
     std::unique_ptr<ExtractStrategy> m_strategy;
     std::vector<std::unique_ptr<Extract>> m_extracts;
@@ -59,7 +59,9 @@ class CommandExtract : public Command, public with_single_osm_input, public with
 
 public:
 
-    CommandExtract() = default;
+    explicit CommandExtract(const CommandFactory& command_factory) :
+        Command(command_factory) {
+    }
 
     bool setup(const std::vector<std::string>& arguments) override final;
 
