@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,6 +20,31 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "command_fileinfo.hpp"
+#include "exception.hpp"
+#include "util.hpp"
+
+#include <osmium/handler.hpp>
+#include <osmium/io/file.hpp>
+#include <osmium/io/header.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/osm.hpp>
+#include <osmium/osm/box.hpp>
+#include <osmium/osm/crc.hpp>
+#include <osmium/osm/crc_zlib.hpp>
+#include <osmium/osm/metadata_options.hpp>
+#include <osmium/osm/object_comparisons.hpp>
+#include <osmium/util/file.hpp>
+#include <osmium/util/minmax.hpp>
+#include <osmium/util/progress_bar.hpp>
+#include <osmium/util/verbose_output.hpp>
+#include <osmium/visitor.hpp>
+
+#include <rapidjson/prettywriter.h>
+#include <rapidjson/stringbuffer.h>
+
+#include <boost/program_options.hpp>
+
 #include <algorithm>
 #include <cstdint>
 #include <iostream>
@@ -34,33 +59,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 # include <unistd.h>
 #endif
 
-#pragma GCC diagnostic push
-#pragma GCC diagnostic ignored "-Wold-style-cast"
-#include <rapidjson/prettywriter.h>
-#include <rapidjson/stringbuffer.h>
-#pragma GCC diagnostic pop
-
-#include <boost/crc.hpp>
-#include <boost/program_options.hpp>
-
-#include <osmium/handler.hpp>
-#include <osmium/io/file.hpp>
-#include <osmium/io/header.hpp>
-#include <osmium/io/reader.hpp>
-#include <osmium/osm.hpp>
-#include <osmium/osm/box.hpp>
-#include <osmium/osm/crc.hpp>
-#include <osmium/osm/object_comparisons.hpp>
-#include <osmium/util/file.hpp>
-#include <osmium/util/minmax.hpp>
-#include <osmium/util/progress_bar.hpp>
-#include <osmium/util/verbose_output.hpp>
-#include <osmium/visitor.hpp>
-
-#include "command_fileinfo.hpp"
-#include "exception.hpp"
-#include "util.hpp"
-
 /*************************************************************************/
 
 struct InfoHandler : public osmium::handler::Handler {
@@ -72,22 +70,35 @@ struct InfoHandler : public osmium::handler::Handler {
     uint64_t ways       = 0;
     uint64_t relations  = 0;
 
-    osmium::max_op<osmium::object_id_type> largest_changeset_id{0};
-    osmium::max_op<osmium::object_id_type> largest_node_id{0};
-    osmium::max_op<osmium::object_id_type> largest_way_id{0};
-    osmium::max_op<osmium::object_id_type> largest_relation_id{0};
+    osmium::min_op<osmium::object_id_type> smallest_changeset_id{};
+    osmium::min_op<osmium::object_id_type> smallest_node_id{};
+    osmium::min_op<osmium::object_id_type> smallest_way_id{};
+    osmium::min_op<osmium::object_id_type> smallest_relation_id{};
+
+    osmium::max_op<osmium::object_id_type> largest_changeset_id{};
+    osmium::max_op<osmium::object_id_type> largest_node_id{};
+    osmium::max_op<osmium::object_id_type> largest_way_id{};
+    osmium::max_op<osmium::object_id_type> largest_relation_id{};
+
+    osmium::metadata_options metadata_all_objects{"all"};
+    osmium::metadata_options metadata_some_objects{"none"};
 
     osmium::min_op<osmium::Timestamp> first_timestamp;
     osmium::max_op<osmium::Timestamp> last_timestamp;
 
-    osmium::CRC<boost::crc_32_type> crc32;
+    osmium::CRC<osmium::CRC_zlib> crc32;
 
     bool ordered = true;
     bool multiple_versions = false;
+    bool calculate_crc = false;
 
     osmium::item_type last_type = osmium::item_type::undefined;
     osmium::object_id_type last_id = 0;
 
+    explicit InfoHandler(bool with_crc) :
+        calculate_crc(with_crc) {
+    }
+
     void changeset(const osmium::Changeset& changeset) {
         if (last_type == osmium::item_type::changeset) {
             if (last_id > changeset.id()) {
@@ -98,9 +109,12 @@ struct InfoHandler : public osmium::handler::Handler {
         }
 
         last_id = changeset.id();
-        crc32.update(changeset);
+        if (calculate_crc) {
+            crc32.update(changeset);
+        }
         ++changesets;
 
+        smallest_changeset_id.update(changeset.id());
         largest_changeset_id.update(changeset.id());
     }
 
@@ -108,6 +122,9 @@ struct InfoHandler : public osmium::handler::Handler {
         first_timestamp.update(object.timestamp());
         last_timestamp.update(object.timestamp());
 
+        metadata_all_objects &= osmium::detect_available_metadata(object);
+        metadata_some_objects |= osmium::detect_available_metadata(object);
+
         if (last_type == object.type()) {
             if (last_id == object.id()) {
                 multiple_versions = true;
@@ -124,24 +141,33 @@ struct InfoHandler : public osmium::handler::Handler {
     }
 
     void node(const osmium::Node& node) {
-        crc32.update(node);
+        if (calculate_crc) {
+            crc32.update(node);
+        }
         bounds.extend(node.location());
         ++nodes;
 
+        smallest_node_id.update(node.id());
         largest_node_id.update(node.id());
     }
 
     void way(const osmium::Way& way) {
-        crc32.update(way);
+        if (calculate_crc) {
+            crc32.update(way);
+        }
         ++ways;
 
+        smallest_way_id.update(way.id());
         largest_way_id.update(way.id());
     }
 
     void relation(const osmium::Relation& relation) {
-        crc32.update(relation);
+        if (calculate_crc) {
+            crc32.update(relation);
+        }
         ++relations;
 
+        smallest_relation_id.update(relation.id());
         largest_relation_id.update(relation.id());
     }
 
@@ -149,9 +175,25 @@ struct InfoHandler : public osmium::handler::Handler {
 
 class Output {
 
+protected:
+
+    bool m_calculate_crc = false;
+
 public:
 
-    virtual ~Output() = default;
+    Output() noexcept = default;
+
+    virtual ~Output() noexcept = default;
+
+    Output(const Output&) = delete;
+    Output& operator=(const Output&) = delete;
+
+    Output(Output&&) noexcept = delete;
+    Output& operator=(Output&&) noexcept = delete;
+
+    void set_crc(bool with_crc) noexcept {
+        m_calculate_crc = with_crc;
+    }
 
     virtual void file(const std::string& filename, const osmium::io::File& input_file) = 0;
     virtual void header(const osmium::io::Header& header) = 0;
@@ -162,22 +204,30 @@ public:
 
 }; // class Output
 
+static osmium::object_id_type get_smallest(osmium::object_id_type id) noexcept {
+    return id == osmium::min_op<osmium::object_id_type>{}() ? 0 : id;
+}
+
+static osmium::object_id_type get_largest(osmium::object_id_type id) noexcept {
+    return id == osmium::max_op<osmium::object_id_type>{}() ? 0 : id;
+}
+
 class HumanReadableOutput : public Output {
 
 public:
 
-    void file(const std::string& input_filename, const osmium::io::File& input_file) override final {
+    void file(const std::string& input_filename, const osmium::io::File& input_file) final {
         std::cout << "File:\n";
         std::cout << "  Name: " << input_filename << "\n";
         std::cout << "  Format: " << input_file.format() << "\n";
         std::cout << "  Compression: " << input_file.compression() << "\n";
 
         if (!input_file.filename().empty()) {
-            std::cout << "  Size: " << osmium::util::file_size(input_file.filename()) << "\n";
+            std::cout << "  Size: " << osmium::file_size(input_file.filename()) << "\n";
         }
     }
 
-    void header(const osmium::io::Header& header) override final {
+    void header(const osmium::io::Header& header) final {
         std::cout << "Header:\n";
 
         std::cout << "  Bounding boxes:\n";
@@ -192,7 +242,7 @@ public:
         }
     }
 
-    void data(const osmium::io::Header& header, const InfoHandler& info_handler) override final {
+    void data(const osmium::io::Header& header, const InfoHandler& info_handler) final {
         std::cout << "Data:\n";
         std::cout << "  Bounding box: " << info_handler.bounds << "\n";
 
@@ -214,22 +264,33 @@ public:
             std::cout << "unknown (because objects in file are unordered)\n";
         }
 
-        std::cout << "  CRC32: " << std::hex << info_handler.crc32().checksum() << std::dec << "\n";
+        if (m_calculate_crc) {
+            std::cout << "  CRC32: " << std::hex << info_handler.crc32().checksum() << std::dec << "\n";
+        } else {
+            std::cout << "  CRC32: not calculated (use --crc/-c to enable)\n";
+        }
 
         std::cout << "  Number of changesets: " << info_handler.changesets << "\n";
         std::cout << "  Number of nodes: "      << info_handler.nodes      << "\n";
         std::cout << "  Number of ways: "       << info_handler.ways       << "\n";
         std::cout << "  Number of relations: "  << info_handler.relations  << "\n";
 
-        std::cout << "  Largest changeset ID: " << info_handler.largest_changeset_id() << "\n";
-        std::cout << "  Largest node ID: "      << info_handler.largest_node_id()      << "\n";
-        std::cout << "  Largest way ID: "       << info_handler.largest_way_id()       << "\n";
-        std::cout << "  Largest relation ID: "  << info_handler.largest_relation_id()  << "\n";
+        std::cout << "  Smallest changeset ID: " << get_smallest(info_handler.smallest_changeset_id()) << "\n";
+        std::cout << "  Smallest node ID: "      << get_smallest(info_handler.smallest_node_id())      << "\n";
+        std::cout << "  Smallest way ID: "       << get_smallest(info_handler.smallest_way_id())       << "\n";
+        std::cout << "  Smallest relation ID: "  << get_smallest(info_handler.smallest_relation_id())  << "\n";
+
+        std::cout << "  Largest changeset ID: " << get_largest(info_handler.largest_changeset_id()) << "\n";
+        std::cout << "  Largest node ID: "      << get_largest(info_handler.largest_node_id())      << "\n";
+        std::cout << "  Largest way ID: "       << get_largest(info_handler.largest_way_id())       << "\n";
+        std::cout << "  Largest relation ID: "  << get_largest(info_handler.largest_relation_id())  << "\n";
+
+        std::cout << "  All objects have following metadata attributes: " << info_handler.metadata_all_objects  << "\n";
+        std::cout << "  Some objects have following metadata attributes: " << info_handler.metadata_some_objects  << "\n";
     }
 
 }; // class HumanReadableOutput
 
-
 class JSONOutput : public Output {
 
     using writer_type = rapidjson::PrettyWriter<rapidjson::StringBuffer>;
@@ -240,12 +301,11 @@ class JSONOutput : public Output {
 public:
 
     JSONOutput() :
-        m_stream(),
         m_writer(m_stream) {
         m_writer.StartObject();
     }
 
-    void file(const std::string& input_filename, const osmium::io::File& input_file) override final {
+    void file(const std::string& input_filename, const osmium::io::File& input_file) final {
         m_writer.String("file");
         m_writer.StartObject();
 
@@ -260,7 +320,7 @@ public:
 
         if (!input_file.filename().empty()) {
             m_writer.String("size");
-            m_writer.Int64(osmium::util::file_size(input_file.filename()));
+            m_writer.Int64(osmium::file_size(input_file.filename()));
         }
 
         m_writer.EndObject();
@@ -275,7 +335,7 @@ public:
         m_writer.EndArray();
     }
 
-    void header(const osmium::io::Header& header) override final {
+    void header(const osmium::io::Header& header) final {
         m_writer.String("header");
         m_writer.StartObject();
 
@@ -300,7 +360,7 @@ public:
         m_writer.EndObject();
     }
 
-    void data(const osmium::io::Header& /*header*/, const InfoHandler& info_handler) override final {
+    void data(const osmium::io::Header& /*header*/, const InfoHandler& info_handler) final {
         m_writer.String("data");
         m_writer.StartObject();
 
@@ -329,10 +389,12 @@ public:
             m_writer.Bool(info_handler.multiple_versions);
         }
 
-        m_writer.String("crc32");
-        std::stringstream ss;
-        ss << std::hex << info_handler.crc32().checksum() << std::dec;
-        m_writer.String(ss.str().c_str());
+        if (m_calculate_crc) {
+            m_writer.String("crc32");
+            std::stringstream ss;
+            ss << std::hex << info_handler.crc32().checksum() << std::dec;
+            m_writer.String(ss.str().c_str());
+        }
 
         m_writer.String("count");
         m_writer.StartObject();
@@ -346,22 +408,64 @@ public:
         m_writer.Int64(info_handler.relations);
         m_writer.EndObject();
 
+        m_writer.String("minid");
+        m_writer.StartObject();
+        m_writer.String("changesets");
+        m_writer.Int64(get_smallest(info_handler.smallest_changeset_id()));
+        m_writer.String("nodes");
+        m_writer.Int64(get_smallest(info_handler.smallest_node_id()));
+        m_writer.String("ways");
+        m_writer.Int64(get_smallest(info_handler.smallest_way_id()));
+        m_writer.String("relations");
+        m_writer.Int64(get_smallest(info_handler.smallest_relation_id()));
+        m_writer.EndObject();
+
         m_writer.String("maxid");
         m_writer.StartObject();
         m_writer.String("changesets");
-        m_writer.Int64(info_handler.largest_changeset_id());
+        m_writer.Int64(get_largest(info_handler.largest_changeset_id()));
         m_writer.String("nodes");
-        m_writer.Int64(info_handler.largest_node_id());
+        m_writer.Int64(get_largest(info_handler.largest_node_id()));
         m_writer.String("ways");
-        m_writer.Int64(info_handler.largest_way_id());
+        m_writer.Int64(get_largest(info_handler.largest_way_id()));
         m_writer.String("relations");
-        m_writer.Int64(info_handler.largest_relation_id());
+        m_writer.Int64(get_largest(info_handler.largest_relation_id()));
+        m_writer.EndObject();
+
+        m_writer.String("metadata");
+        m_writer.StartObject();
+        m_writer.String("all_objects");
+        m_writer.StartObject();
+        m_writer.String("version");
+        m_writer.Bool(info_handler.metadata_all_objects.version());
+        m_writer.String("timestamp");
+        m_writer.Bool(info_handler.metadata_all_objects.timestamp());
+        m_writer.String("changeset");
+        m_writer.Bool(info_handler.metadata_all_objects.changeset());
+        m_writer.String("user");
+        m_writer.Bool(info_handler.metadata_all_objects.user());
+        m_writer.String("uid");
+        m_writer.Bool(info_handler.metadata_all_objects.uid());
+        m_writer.EndObject();
+        m_writer.String("some_objects");
+        m_writer.StartObject();
+        m_writer.String("version");
+        m_writer.Bool(info_handler.metadata_some_objects.version());
+        m_writer.String("timestamp");
+        m_writer.Bool(info_handler.metadata_some_objects.timestamp());
+        m_writer.String("changeset");
+        m_writer.Bool(info_handler.metadata_some_objects.changeset());
+        m_writer.String("user");
+        m_writer.Bool(info_handler.metadata_some_objects.user());
+        m_writer.String("uid");
+        m_writer.Bool(info_handler.metadata_some_objects.uid());
+        m_writer.EndObject();
         m_writer.EndObject();
 
         m_writer.EndObject();
     }
 
-    void output() override final {
+    void output() final {
         m_writer.EndObject();
         std::cout << m_stream.GetString() << "\n";
     }
@@ -374,32 +478,37 @@ class SimpleOutput : public Output {
 
 public:
 
-    explicit SimpleOutput(const std::string& get_value) :
-        m_get_value(get_value) {
+    explicit SimpleOutput(std::string get_value) :
+        m_get_value(std::move(get_value)) {
     }
 
-    void file(const std::string& input_filename, const osmium::io::File& input_file) override final {
+    void file(const std::string& input_filename, const osmium::io::File& input_file) final {
         if (m_get_value == "file.name") {
             std::cout << input_filename << "\n";
+            return;
         }
         if (m_get_value == "file.format") {
             std::cout << input_file.format() << "\n";
+            return;
         }
         if (m_get_value == "file.compression") {
             std::cout << input_file.compression() << "\n";
+            return;
         }
         if (m_get_value == "file.size") {
             if (input_file.filename().empty()) {
                 std::cout << 0 << "\n";
             } else {
-                std::cout << osmium::util::file_size(input_file.filename()) << "\n";
+                std::cout << osmium::file_size(input_file.filename()) << "\n";
             }
+            return;
         }
     }
 
-    void header(const osmium::io::Header& header) override final {
+    void header(const osmium::io::Header& header) final {
         if (m_get_value == "header.with_history") {
             std::cout << yes_no(header.has_multiple_object_versions());
+            return;
         }
 
         for (const auto& option : header) {
@@ -411,9 +520,10 @@ public:
         }
     }
 
-    void data(const osmium::io::Header& /*header*/, const InfoHandler& info_handler) override final {
+    void data(const osmium::io::Header& /*header*/, const InfoHandler& info_handler) final {
         if (m_get_value == "data.bbox") {
             std::cout << info_handler.bounds << "\n";
+            return;
         }
 
         if (m_get_value == "data.timestamp.first") {
@@ -422,6 +532,7 @@ public:
             } else {
                 std::cout << info_handler.first_timestamp() << "\n";
             }
+            return;
         }
 
         if (m_get_value == "data.timestamp.last") {
@@ -430,10 +541,12 @@ public:
             } else {
                 std::cout << info_handler.last_timestamp() << "\n";
             }
+            return;
         }
 
         if (m_get_value == "data.objects_ordered") {
             std::cout << (info_handler.ordered ? "yes\n" : "no\n");
+            return;
         }
 
         if (m_get_value == "data.multiple_versions") {
@@ -442,36 +555,104 @@ public:
             } else {
                 std::cout << "unknown\n";
             }
+            return;
         }
 
         if (m_get_value == "data.crc32") {
             std::cout << std::hex << info_handler.crc32().checksum() << std::dec << "\n";
+            return;
         }
 
         if (m_get_value == "data.count.changesets") {
             std::cout << info_handler.changesets << "\n";
+            return;
         }
         if (m_get_value == "data.count.nodes") {
             std::cout << info_handler.nodes << "\n";
+            return;
         }
         if (m_get_value == "data.count.ways") {
             std::cout << info_handler.ways << "\n";
+            return;
         }
         if (m_get_value == "data.count.relations") {
             std::cout << info_handler.relations << "\n";
+            return;
+        }
+
+        if (m_get_value == "data.minid.changesets") {
+            std::cout << get_smallest(info_handler.smallest_changeset_id()) << "\n";
+            return;
+        }
+        if (m_get_value == "data.minid.nodes") {
+            std::cout << get_smallest(info_handler.smallest_node_id()) << "\n";
+            return;
+        }
+        if (m_get_value == "data.minid.ways") {
+            std::cout << get_smallest(info_handler.smallest_way_id()) << "\n";
+            return;
+        }
+        if (m_get_value == "data.minid.relations") {
+            std::cout << get_smallest(info_handler.smallest_relation_id()) << "\n";
+            return;
         }
 
         if (m_get_value == "data.maxid.changesets") {
-            std::cout << info_handler.largest_changeset_id() << "\n";
+            std::cout << get_largest(info_handler.largest_changeset_id()) << "\n";
+            return;
         }
         if (m_get_value == "data.maxid.nodes") {
-            std::cout << info_handler.largest_node_id() << "\n";
+            std::cout << get_largest(info_handler.largest_node_id()) << "\n";
+            return;
         }
         if (m_get_value == "data.maxid.ways") {
-            std::cout << info_handler.largest_way_id() << "\n";
+            std::cout << get_largest(info_handler.largest_way_id()) << "\n";
+            return;
         }
         if (m_get_value == "data.maxid.relations") {
-            std::cout << info_handler.largest_relation_id() << "\n";
+            std::cout << get_largest(info_handler.largest_relation_id()) << "\n";
+            return;
+        }
+
+        if (m_get_value == "metadata.all_objects.version") {
+            std::cout << (info_handler.metadata_all_objects.version() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.all_objects.timestamp") {
+            std::cout << (info_handler.metadata_all_objects.timestamp() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.all_objects.changeset") {
+            std::cout << (info_handler.metadata_all_objects.changeset() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.all_objects.uid") {
+            std::cout << (info_handler.metadata_all_objects.uid() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.all_objects.user") {
+            std::cout << (info_handler.metadata_all_objects.user() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.some_objects.version") {
+            std::cout << (info_handler.metadata_some_objects.version() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.some_objects.timestamp") {
+            std::cout << (info_handler.metadata_some_objects.timestamp() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.some_objects.changeset") {
+            std::cout << (info_handler.metadata_some_objects.changeset() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.some_objects.uid") {
+            std::cout << (info_handler.metadata_some_objects.uid() ? "yes\n" : "no\n");
+            return;
+        }
+        if (m_get_value == "metadata.some_objects.user") {
+            std::cout << (info_handler.metadata_some_objects.user() ? "yes\n" : "no\n");
+            return;
         }
     }
 
@@ -487,6 +668,8 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
     ("get,g", po::value<std::string>(), "Get value")
     ("show-variables,G", "Show variables for --get option")
     ("json,j", "JSON output")
+    ("crc,c", "Calculate CRC")
+    ("no-crc", "Do not calculate CRC")
     ;
 
     po::options_description opts_common{add_common_options()};
@@ -521,7 +704,15 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
         m_json_output = true;
     }
 
-    std::vector<std::string> known_values = {
+    if (vm.count("crc") && vm.count("no-crc")) {
+        throw argument_error{"Can not use --crc/-c option and --no-crc at the same time."};
+    }
+
+    if (m_extended && (vm.count("crc") || (m_json_output && !vm.count("no-crc")))) {
+        m_calculate_crc = true;
+    }
+
+    const std::vector<std::string> known_values = {
         "file.name",
         "file.format",
         "file.compression",
@@ -544,19 +735,33 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
         "data.count.ways",
         "data.count.relations",
         "data.count.changesets",
+        "data.minid.nodes",
+        "data.minid.ways",
+        "data.minid.relations",
+        "data.minid.changesets",
         "data.maxid.nodes",
         "data.maxid.ways",
         "data.maxid.relations",
-        "data.maxid.changesets"
+        "data.maxid.changesets",
+        "metadata.all_objects.version",
+        "metadata.all_objects.timestamp",
+        "metadata.all_objects.changeset",
+        "metadata.all_objects.uid",
+        "metadata.all_objects.user",
+        "metadata.some_objects.version",
+        "metadata.some_objects.timestamp",
+        "metadata.some_objects.changeset",
+        "metadata.some_objects.uid",
+        "metadata.some_objects.user"
     };
 
     if (vm.count("show-variables")) {
         std::copy(known_values.cbegin(), known_values.cend(), std::ostream_iterator<std::string>(std::cout, "\n"));
         return false;
-    } else {
-        setup_input_file(vm);
     }
 
+    setup_input_file(vm);
+
     if (vm.count("get")) {
         m_get_value = vm["get"].as<std::string>();
         if (m_get_value.substr(0, 14) != "header.option.") {
@@ -565,9 +770,14 @@ bool CommandFileinfo::setup(const std::vector<std::string>& arguments) {
                 throw argument_error{std::string{"Unknown value for --get/-g option '"} + m_get_value + "'. Use --show-variables/-G to see list of known values."};
             }
         }
-        if (m_get_value.substr(0, 5) == "data." && ! m_extended) {
+        if (m_get_value.substr(0, 5) == "data." && !m_extended) {
             throw argument_error{"You need to set --extended/-e for any 'data.*' variables to be available."};
         }
+        if (m_get_value == "data.crc32") {
+            m_calculate_crc = true;
+        } else {
+            m_calculate_crc = false;
+        }
     }
 
     if (vm.count("get") && vm.count("json")) {
@@ -582,18 +792,20 @@ void CommandFileinfo::show_arguments() {
 
     m_vout << "  other options:\n";
     m_vout << "    extended output: " << (m_extended ? "yes\n" : "no\n");
+    m_vout << "    calculate CRC: " << (m_calculate_crc ? "yes\n" : "no\n");
 }
 
 bool CommandFileinfo::run() {
     std::unique_ptr<Output> output;
     if (m_json_output) {
-        output.reset(new JSONOutput());
+        output.reset(new JSONOutput{});
     } else if (m_get_value.empty()) {
-        output.reset(new HumanReadableOutput());
+        output.reset(new HumanReadableOutput{});
     } else {
-        output.reset(new SimpleOutput(m_get_value));
+        output.reset(new SimpleOutput{m_get_value});
     }
 
+    output->set_crc(m_calculate_crc);
     output->file(m_input_filename, m_input_file);
 
     osmium::io::Reader reader{m_input_file, m_extended ? osmium::osm_entity_bits::all : osmium::osm_entity_bits::nothing};
@@ -601,7 +813,7 @@ bool CommandFileinfo::run() {
     output->header(header);
 
     if (m_extended) {
-        InfoHandler info_handler;
+        InfoHandler info_handler{m_calculate_crc};
         osmium::ProgressBar progress_bar{reader.file_size(), display_progress()};
         while (osmium::memory::Buffer buffer = reader.read()) {
             progress_bar.update(reader.offset());
