@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,20 +23,23 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
+#include "cmd.hpp" // IWYU pragma: export
+
 #include <string>
 #include <vector>
 
-#include "cmd.hpp" // IWYU pragma: export
-
 class CommandFileinfo : public Command, public with_single_osm_input {
 
     bool m_extended = false;
     bool m_json_output = false;
+    bool m_calculate_crc = false;
     std::string m_get_value;
 
 public:
 
-    CommandFileinfo() = default;
+    explicit CommandFileinfo(const CommandFactory& command_factory) :
+        Command(command_factory) {
+    }
 
     bool setup(const std::vector<std::string>& arguments) override final;
 
