@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,10 +23,7 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <cstddef>
-#include <iosfwd>
-#include <string>
-#include <vector>
+#include "cmd.hpp" // IWYU pragma: export
 
 #include <osmium/fwd.hpp>
 #include <osmium/index/id_set.hpp>
@@ -35,11 +32,18 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/osm/item_type.hpp>
 #include <osmium/osm/types.hpp>
 
-#include "cmd.hpp" // IWYU pragma: export
+#include <cstddef>
+#include <iosfwd>
+#include <string>
+#include <vector>
 
-namespace osmium { namespace index {
-    class RelationsMapIndex;
-}}
+namespace osmium {
+
+    namespace index {
+        class RelationsMapIndex;
+    } // namespace index
+
+} // namespace osmium
 
 class CommandGetId : public Command, public with_single_osm_input, public with_osm_output {
 
@@ -51,20 +55,11 @@ class CommandGetId : public Command, public with_single_osm_input, public with_o
 
     osmium::nwr_array<osmium::index::IdSetDense<osmium::unsigned_object_id_type>> m_ids;
 
-    void parse_and_add_id(const std::string& s);
-
-    void read_id_osm_file(const std::string& file_name);
-    void read_id_file(std::istream& stream);
-
     osmium::osm_entity_bits::type get_needed_types() const;
-    bool no_ids() const;
-    std::size_t count_ids() const;
+    std::size_t count_ids() const noexcept;
 
     void find_referenced_objects();
 
-    void add_nodes(const osmium::Way& way);
-    void add_members(const osmium::Relation& relation);
-
     void mark_rel_ids(const osmium::index::RelationsMapIndex& rel_in_rel, osmium::object_id_type parent_id);
     bool find_relations_in_relations();
     void find_nodes_and_ways_in_relations();
@@ -72,7 +67,9 @@ class CommandGetId : public Command, public with_single_osm_input, public with_o
 
 public:
 
-    CommandGetId() = default;
+    explicit CommandGetId(const CommandFactory& command_factory) :
+        Command(command_factory) {
+    }
 
     bool setup(const std::vector<std::string>& arguments) override final;
 
