@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,8 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <iostream>
+#include "command_help.hpp"
+
 #include <iomanip>
+#include <iostream>
 #include <string>
 #include <vector>
 
@@ -29,8 +31,6 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 # include <unistd.h>
 #endif
 
-#include "command_help.hpp"
-
 bool CommandHelp::setup(const std::vector<std::string>& arguments) {
     m_topic = arguments.empty() ? "help" : arguments.front();
     return true;
@@ -46,11 +46,11 @@ static void show_help(const std::string& topic, const std::string& info) {
 #endif
     // show info string and link on Windows systems
     std::cout << info << "\n";
-    std::cout << "You'll find more documentation at http://osmcode.org/osmium-tool/\n";
+    std::cout << "You'll find more documentation at https://osmcode.org/osmium-tool/\n";
 }
 
 bool CommandHelp::run() {
-    auto commands = CommandFactory::help();
+    auto commands = m_command_factory.help();
 
     if (m_topic == "help") {
         std::cout << "Usage: " << synopsis()
@@ -59,7 +59,7 @@ bool CommandHelp::run() {
         // print command names and descriptions in a nice table
         for (const auto& cmd : commands) {
             std::cout << "  "
-                      << std::setw(CommandFactory::max_command_name_length())
+                      << std::setw(m_command_factory.max_command_name_length())
                       << std::left
                       << cmd.first
                       << std::setw(0)
@@ -78,7 +78,7 @@ bool CommandHelp::run() {
         return true;
     }
 
-    std::string description = CommandFactory::get_description(m_topic);
+    std::string description = m_command_factory.get_description(m_topic);
     if (!description.empty()) {
         show_help(m_topic, std::string{"osmium "} + m_topic + ": " + description);
         return true;
