@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,30 +20,37 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <algorithm>
-#include <memory>
-#include <queue>
-#include <string>
-#include <vector>
-#include <utility>
-
-#include <boost/program_options.hpp>
+#include "command_merge.hpp"
+#include "util.hpp"
 
 #include <osmium/io/header.hpp>
-#include <osmium/io/reader.hpp>
-#include <osmium/io/writer.hpp>
 #include <osmium/io/input_iterator.hpp>
 #include <osmium/io/output_iterator.hpp>
+#include <osmium/io/reader.hpp>
+#include <osmium/io/reader_with_progress_bar.hpp>
+#include <osmium/io/writer.hpp>
 #include <osmium/memory/buffer.hpp>
 #include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/object.hpp>
 #include <osmium/util/verbose_output.hpp>
 
-#include "command_merge.hpp"
+#include <boost/program_options.hpp>
 
-namespace osmium { namespace io {
-    class File;
-}}
+#include <algorithm>
+#include <memory>
+#include <numeric>
+#include <queue>
+#include <string>
+#include <utility>
+#include <vector>
+
+namespace osmium {
+
+    namespace io {
+        class File;
+    } // namespace io
+
+} // namespace osmium
 
 bool CommandMerge::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_cmd{"COMMAND OPTIONS"};
@@ -71,6 +78,7 @@ bool CommandMerge::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_files(vm);
     setup_output_file(vm);
 
@@ -111,6 +119,10 @@ namespace {
             return &*iterator;
         }
 
+        std::size_t offset() const noexcept {
+            return reader->offset();
+        }
+
     }; // DataSource
 
     class QueueElement {
@@ -144,7 +156,7 @@ namespace {
     }
 
     bool operator!=(const QueueElement& lhs, const QueueElement& rhs) noexcept {
-        return ! (lhs == rhs);
+        return !(lhs == rhs);
     }
 
 } // anonymous namespace
@@ -158,14 +170,22 @@ bool CommandMerge::run() {
 
     if (m_input_files.size() == 1) {
         m_vout << "Single input file. Copying to output file...\n";
-        osmium::io::Reader reader{m_input_files[0]};
+        osmium::io::ReaderWithProgressBar reader{display_progress(), m_input_files[0]};
         while (osmium::memory::Buffer buffer = reader.read()) {
             writer(std::move(buffer));
         }
     } else if (m_input_files.size() == 2) {
         // Use simpler code when there are exactly two files to merge
         m_vout << "Merging 2 input files to output file...\n";
-        osmium::io::Reader reader1(m_input_files[0], osmium::osm_entity_bits::object);
+
+        // The larger file should be first so the progress bar will work better
+        if (osmium::file_size(m_input_files[0].filename()) <
+            osmium::file_size(m_input_files[1].filename())) {
+            using std::swap;
+            swap(m_input_files[0], m_input_files[1]);
+        }
+
+        osmium::io::ReaderWithProgressBar reader1(display_progress(), m_input_files[0], osmium::osm_entity_bits::object);
         osmium::io::Reader reader2(m_input_files[1], osmium::osm_entity_bits::object);
         auto in1 = osmium::io::make_input_iterator_range<osmium::OSMObject>(reader1);
         auto in2 = osmium::io::make_input_iterator_range<osmium::OSMObject>(reader2);
@@ -177,6 +197,7 @@ bool CommandMerge::run() {
     } else {
         // Three or more files to merge
         m_vout << "Merging " << m_input_files.size() << " input files to output file...\n";
+        osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
         std::vector<DataSource> data_sources;
         data_sources.reserve(m_input_files.size());
 
@@ -193,8 +214,9 @@ bool CommandMerge::run() {
             ++index;
         }
 
+        int n = 0;
         while (!queue.empty()) {
-            auto element = queue.top();
+            const auto element = queue.top();
             queue.pop();
             if (queue.empty() || element != queue.top()) {
                 writer(element.object());
@@ -204,6 +226,13 @@ bool CommandMerge::run() {
             if (data_sources[index].next()) {
                 queue.emplace(data_sources[index].get(), index);
             }
+
+            if (n++ > 10000) {
+                n = 0;
+                progress_bar.update(std::accumulate(data_sources.cbegin(), data_sources.cend(), static_cast<std::size_t>(0), [](std::size_t sum, const DataSource& source){
+                    return sum + source.offset();
+                }));
+            }
         }
     }
 
