@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,12 +20,8 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <algorithm>
-#include <string>
-#include <utility>
-#include <vector>
-
-#include <boost/program_options.hpp>
+#include "command_sort.hpp"
+#include "util.hpp"
 
 #include <osmium/io/header.hpp>
 #include <osmium/io/output_iterator.hpp>
@@ -36,10 +32,16 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/osm/box.hpp>
 #include <osmium/osm/entity_bits.hpp>
 #include <osmium/osm/object_comparisons.hpp>
+#include <osmium/util/progress_bar.hpp>
 #include <osmium/util/verbose_output.hpp>
 #include <osmium/visitor.hpp>
 
-#include "command_sort.hpp"
+#include <boost/program_options.hpp>
+
+#include <algorithm>
+#include <string>
+#include <utility>
+#include <vector>
 
 bool CommandSort::setup(const std::vector<std::string>& arguments) {
     po::options_description opts_common{add_common_options()};
@@ -65,6 +67,7 @@ bool CommandSort::setup(const std::vector<std::string>& arguments) {
     po::notify(vm);
 
     setup_common(vm, desc);
+    setup_progress(vm);
     setup_input_files(vm);
     setup_output_file(vm);
 
@@ -87,16 +90,20 @@ bool CommandSort::run() {
     osmium::Box bounding_box;
 
     m_vout << "Reading contents of input files...\n";
+    osmium::ProgressBar progress_bar{file_size_sum(m_input_files), display_progress()};
     for (const std::string& file_name : m_filenames) {
         osmium::io::Reader reader{file_name, osmium::osm_entity_bits::object};
         osmium::io::Header header{reader.header()};
         bounding_box.extend(header.joined_boxes());
         while (osmium::memory::Buffer buffer = reader.read()) {
+            progress_bar.update(reader.offset());
             osmium::apply(buffer, objects);
             data.push_back(std::move(buffer));
         }
+        progress_bar.file_done(reader.file_size());
         reader.close();
     }
+    progress_bar.done();
 
     m_vout << "Opening output file...\n";
     osmium::io::Header header;
