@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,10 +20,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <osmium/io/detail/read_write.hpp>
-
 #include "export_format_json.hpp"
 
+#include <osmium/io/detail/read_write.hpp>
+
 static constexpr const std::size_t initial_buffer_size = 1024 * 1024;
 static constexpr const std::size_t flush_buffer_size   =  800 * 1024;
 
@@ -43,8 +43,6 @@ ExportFormatJSON::ExportFormatJSON(const std::string& output_format,
     m_fsync(fsync),
     m_text_sequence_format(output_format == "geojsonseq"),
     m_with_record_separator(m_text_sequence_format && options.print_record_separator),
-    m_stream(),
-    m_committed_size(0),
     m_writer(m_stream),
     m_factory(m_writer) {
     m_stream.Reserve(initial_buffer_size);
@@ -128,7 +126,7 @@ void ExportFormatJSON::add_attributes(const osmium::OSMObject& object) {
 
     if (!options().timestamp.empty()) {
         m_writer.String(options().timestamp);
-        m_writer.Int(object.timestamp().seconds_since_epoch());
+        m_writer.Int64(object.timestamp().seconds_since_epoch());
     }
 
     if (!options().way_nodes.empty() && object.type() == osmium::item_type::way) {
@@ -141,27 +139,18 @@ void ExportFormatJSON::add_attributes(const osmium::OSMObject& object) {
     }
 }
 
-bool ExportFormatJSON::add_tags(const osmium::OSMObject& object) {
-    bool has_tags = false;
-
-    for (const auto& tag : object.tags()) {
-        if (options().tags_filter(tag)) {
-            has_tags = true;
-            m_writer.String(tag.key());
-            m_writer.String(tag.value());
-        }
-    }
-
-    return has_tags;
-}
-
 void ExportFormatJSON::finish_feature(const osmium::OSMObject& object) {
     m_writer.Key("properties");
     m_writer.StartObject(); // start properties
 
     add_attributes(object);
 
-    if (add_tags(object) || options().keep_untagged) {
+    const bool has_tags = add_tags(object, [&](const osmium::Tag& tag) {
+        m_writer.String(tag.key());
+        m_writer.String(tag.value());
+    });
+
+    if (has_tags || options().keep_untagged) {
         m_writer.EndObject(); // end properties
         m_writer.EndObject(); // end feature
 
