@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,12 +20,12 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <osmium/handler/check_order.hpp>
-#include <osmium/util/string.hpp>
-
 #include "strategy_smart.hpp"
 #include "../util.hpp"
 
+#include <osmium/handler/check_order.hpp>
+#include <osmium/util/string.hpp>
+
 namespace strategy_smart {
 
     void Data::add_relation(const osmium::Relation& relation) {
@@ -46,17 +46,15 @@ namespace strategy_smart {
 
     void Data::add_relation_parents(osmium::unsigned_object_id_type id, const osmium::index::RelationsMapIndex& map) {
         map.for_each_parent(id, [&](osmium::unsigned_object_id_type parent_id) {
-            if (! relation_ids.get(parent_id) &&
-                ! extra_relation_ids.get(parent_id)) {
+            if (!relation_ids.get(parent_id) &&
+                !extra_relation_ids.get(parent_id)) {
                 relation_ids.set(parent_id);
                 add_relation_parents(parent_id, map);
             }
         });
     }
 
-    Strategy::Strategy(const std::vector<std::unique_ptr<Extract>>& extracts, const osmium::util::Options& options) :
-        ExtractStrategy(),
-        m_types() {
+    Strategy::Strategy(const std::vector<std::unique_ptr<Extract>>& extracts, const osmium::Options& options) {
         m_extracts.reserve(extracts.size());
         for (const auto& extract : extracts) {
             m_extracts.emplace_back(*extract);
@@ -69,7 +67,7 @@ namespace strategy_smart {
         }
 
         const auto types = options.get("types");
-        if (types == "") {
+        if (types.empty()) {
             m_types = {"multipolygon"};
         } else if (types != "any") {
             m_types = osmium::split_string(types, ',', true);
@@ -93,7 +91,7 @@ namespace strategy_smart {
         return it != m_types.end();
     }
 
-    void Strategy::show_arguments(osmium::util::VerboseOutput& vout) {
+    void Strategy::show_arguments(osmium::VerboseOutput& vout) {
         vout << "Additional strategy options:\n";
         if (m_types.empty()) {
             vout << "  types: any\n";
@@ -113,7 +111,7 @@ namespace strategy_smart {
 
     public:
 
-        Pass1(Strategy& strategy) :
+        explicit Pass1(Strategy& strategy) :
             Pass(strategy) {
         }
 
@@ -182,7 +180,7 @@ namespace strategy_smart {
 
     public:
 
-        Pass2(Strategy& strategy) :
+        explicit Pass2(Strategy& strategy) :
             Pass(strategy) {
         }
 
@@ -201,7 +199,7 @@ namespace strategy_smart {
 
     public:
 
-        Pass3(Strategy& strategy) :
+        explicit Pass3(Strategy& strategy) :
             Pass(strategy) {
         }
 
@@ -228,12 +226,16 @@ namespace strategy_smart {
 
     }; // class Pass3
 
-    void Strategy::run(osmium::util::VerboseOutput& vout, bool display_progress, const osmium::io::File& input_file) {
+    void Strategy::run(osmium::VerboseOutput& vout, bool display_progress, const osmium::io::File& input_file) {
+        if (input_file.filename().empty()) {
+            throw osmium::io_error{"Can not read from STDIN when using 'smart' strategy."};
+        }
+
         vout << "Running 'smart' strategy in three passes...\n";
-        const std::size_t file_size = osmium::util::file_size(input_file.filename());
+        const std::size_t file_size = osmium::file_size(input_file.filename());
         osmium::ProgressBar progress_bar{file_size * 3, display_progress};
 
-        vout << "First pass...\n";
+        vout << "First pass (of three)...\n";
         Pass1 pass1{*this};
         pass1.run(progress_bar, input_file, osmium::io::read_meta::no);
         progress_bar.file_done(file_size);
@@ -247,13 +249,13 @@ namespace strategy_smart {
         }
 
         progress_bar.remove();
-        vout << "Second pass...\n";
+        vout << "Second pass (of three)...\n";
         Pass2 pass2{*this};
         pass2.run(progress_bar, input_file, osmium::osm_entity_bits::way, osmium::io::read_meta::no);
         progress_bar.file_done(file_size);
 
         progress_bar.remove();
-        vout << "Third pass...\n";
+        vout << "Third pass (of three)...\n";
         Pass3 pass3{*this};
         pass3.run(progress_bar, input_file);
 
