@@ -4,9 +4,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -23,14 +23,14 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <memory>
-#include <string>
-#include <vector>
+#include "strategy.hpp"
 
 #include <osmium/index/id_set.hpp>
 #include <osmium/index/relations_map.hpp>
 
-#include "strategy.hpp"
+#include <memory>
+#include <string>
+#include <vector>
 
 namespace strategy_smart {
 
@@ -60,13 +60,13 @@ namespace strategy_smart {
 
     public:
 
-        explicit Strategy(const std::vector<std::unique_ptr<Extract>>& extracts, const osmium::util::Options& options);
+        explicit Strategy(const std::vector<std::unique_ptr<Extract>>& extracts, const osmium::Options& options);
 
         const char* name() const noexcept override final;
 
-        void show_arguments(osmium::util::VerboseOutput& vout) override final;
+        void show_arguments(osmium::VerboseOutput& vout) override final;
 
-        void run(osmium::util::VerboseOutput& vout, bool display_progress, const osmium::io::File& input_file) override final;
+        void run(osmium::VerboseOutput& vout, bool display_progress, const osmium::io::File& input_file) override final;
 
     }; // class Strategy
 
