@@ -1,9 +1,9 @@
 /*
 
 Osmium -- OpenStreetMap data manipulation command line tool
-http://osmcode.org/osmium-tool/
+https://osmcode.org/osmium-tool/
 
-Copyright (C) 2013-2017  Jochen Topf <jochen@topf.org>
+Copyright (C) 2013-2018  Jochen Topf <jochen@topf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -20,10 +20,9 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
 */
 
-#include <string>
-#include <vector>
-
-#include <boost/program_options.hpp>
+#include "cmd.hpp"
+#include "exception.hpp"
+#include "util.hpp"
 
 #include <osmium/io/any_input.hpp> // IWYU pragma: keep
 #include <osmium/io/any_output.hpp> // IWYU pragma: keep
@@ -32,9 +31,10 @@ along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #include <osmium/io/writer_options.hpp>
 #include <osmium/util/verbose_output.hpp>
 
-#include "cmd.hpp"
-#include "exception.hpp"
-#include "util.hpp"
+#include <boost/program_options.hpp>
+
+#include <string>
+#include <vector>
 
 void with_single_osm_input::setup_input_file(const boost::program_options::variables_map& vm) {
     if (vm.count("input-filename")) {
@@ -51,7 +51,7 @@ void with_single_osm_input::setup_input_file(const boost::program_options::varia
                                  "to specify the file format."};
         }
 
-        if (m_input_filename == "") {
+        if (m_input_filename.empty()) {
             throw argument_error{"Missing input file. Use '-' to read from STDIN and add the --input-format/-F\n"
                                  "option to specify the file format or specify the input file name."};
         }
@@ -70,7 +70,7 @@ po::options_description with_single_osm_input::add_single_input_options() {
     return options;
 }
 
-void with_single_osm_input::show_single_input_arguments(osmium::util::VerboseOutput& vout) {
+void with_single_osm_input::show_single_input_arguments(osmium::VerboseOutput& vout) {
     vout << "  input options:\n";
     vout << "    file name: " << m_input_filename << "\n";
     vout << "    file format: " << m_input_format << "\n";
@@ -80,7 +80,7 @@ void with_multiple_osm_inputs::setup_input_files(const boost::program_options::v
     if (vm.count("input-filenames")) {
         m_input_filenames = vm["input-filenames"].as<std::vector<std::string>>();
     } else {
-        m_input_filenames.push_back("-"); // default is stdin
+        m_input_filenames.emplace_back("-"); // default is stdin
     }
 
     bool uses_stdin = false;
@@ -119,7 +119,7 @@ po::options_description with_multiple_osm_inputs::add_multiple_inputs_options()
     return options;
 }
 
-void with_multiple_osm_inputs::show_multiple_inputs_arguments(osmium::util::VerboseOutput& vout) {
+void with_multiple_osm_inputs::show_multiple_inputs_arguments(osmium::VerboseOutput& vout) {
     vout << "  input options:\n";
     vout << "    file names: \n";
     for (const auto& fn : m_input_filenames) {
@@ -160,7 +160,7 @@ void with_osm_output::check_output_file() {
             throw argument_error{"When writing to STDOUT you need to use the --output-format/-f\n"
                                  "option to specify the file format."};
         }
-        if (m_output_filename == "") {
+        if (m_output_filename.empty()) {
             throw argument_error{"Missing output file. Set the output file with --output/-o and/or\n"
                                  "add the --output-format/-f option to specify the file format."};
         }
@@ -190,7 +190,7 @@ po::options_description with_osm_output::add_output_options() {
     return options;
 }
 
-void with_osm_output::show_output_arguments(osmium::util::VerboseOutput& vout) {
+void with_osm_output::show_output_arguments(osmium::VerboseOutput& vout) {
     vout << "  output options:\n";
     vout << "    file name: " << m_output_filename << "\n";
     vout << "    file format: " << m_output_format << "\n";
